package ajedrez.server.persistencia;


import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;


/* Esta clase contiene todos los mtodos de acceso a la base de datos. */
public class Metodos {
	/* Funcionamiento: desde los mtodos que se quiera acceder a la base de
	 * datos, se crea un objeto "Agente" y se llama al mtodo "getAgente()",
	 * que devuelve una instancia de la clase "Agente". */
	
	
	/**
	 * Devuelve la informacin de contacto de la base de datos.
	 * @return   Devuelve una matriz de cinco componentes:
	 *           [0]=IP, [1]=puerto, [2]=nombreBD, [3]=usuario, [4]=contrasea.
	 */
	public static String[] getInfo() {
		return Agente.getInfo();
	} /* Fin mtodo getInfo */
	
	
	/**
	 * Fija una nueva informacin de contacto con la base de datos.
	 */
	public static void setInfo(String sIP, String sPuerto, String sNombreBD, String sUsuario, String sContrasea) {
		Agente.setInfo(sIP, sPuerto, sNombreBD, sUsuario, sContrasea);
	} /* Fin mtodo setInfo */
	
	
	/**
	 * Comprueba la conexin con la base de datos.
	 */
	public static void comprobarConexion() throws Exception {
		Connection bd = Agente.getAgente().getBD();
		bd.close();
	} /* Fin mtodo comprobarConexion */
	
	
	/**
	 * Obtiene el valor mximo de los identificadores de las partidas.
	 */
	public static int getUltimoId() {
		int nNumero = 0;
		PreparedStatement p = null;
		Connection bd = null;
		ResultSet r;
		String SQL = "SELECT MAX([Id Partida]) FROM [Historial Partidas]";
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL);
			r = p.executeQuery();
			if (r.next()) {
				nNumero = r.getInt(1);
			}
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		return nNumero;
	} /* Fin mtodo getUltimoId */
	
	
	/**
	 * Inserta un jugador nuevo en la base de datos; es decir, lo registra.
	 */
	public static boolean insertaJugador(String sUsuario, String sContrasea) {
		boolean bCorrecto = true;
		PreparedStatement p = null;
		Connection bd = null;
		String SQL = "INSERT INTO Jugador (nombreUsuario, contrasea, [Partidas Jugadas], [Partidas Ganadas], [Partidas Perdidas], [Partidas Tablas], Puntuacion) values (?, ?, ?, ?, ?, ?, ?)";
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL);
			p.setString(1, sUsuario);
			p.setString(2, sContrasea);
			p.setInt(3, 0);
			p.setInt(4, 0);
			p.setInt(5, 0);
			p.setInt(6, 0);
			p.setInt(7, 0);
			p.executeUpdate();
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
			bCorrecto = false;
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		return bCorrecto;
	} /* Fin mtodo insertaJugador */
	
	
	/**
	 * Comprueba en la base de datos si la informacin escrita en la
	 * identificacin es correcta.
	 * @return   Devuelve '0' si los datos del usuario son correctos, '-1' si
	 *           nicamente la contrasea est equivocada y '-2' si el usuario
	 *           no est registrado en el sistema.
	 */
	public static int compruebaUsuario(String sUsuario, String sContrasea) {
		int nValor = 0;
		PreparedStatement p = null;
		Connection bd = null;
		ResultSet r;
		String SQL = "SELECT contrasea FROM Jugador WHERE nombreUsuario = ?;";
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL);
			p.setString(1, sUsuario);
			r = p.executeQuery();
			if (r.next()) {
				String sPasswd = r.getString(1);//.substring(0, r.getString(1).indexOf(" "));
				if (!(sPasswd.equals(sContrasea))) {
					nValor = -1;
				}
			}
			else {
				nValor = -2;
			}
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		return nValor;
	} /* Fin mtodo compruebaUsuario */
	
	
	/**
	 * Inserta el resultado y las estadsticas de una partida que acaba de
	 * terminar en la base de datos.
	 */
	public static void insertaPartida(int idPart, String id1, String id2, int resul, String movimientos) {
		int PJ1 = 0, PG1 = 0, PP1 = 0, PT1 = 0, P1 = 0, PJ2 = 0, PG2 = 0, PP2 = 0, PT2 = 0, P2 = 0;
		Date fecha = new Date();
		String Fecha = fecha.toString();
		PreparedStatement p = null;
		Connection bd = null;
		ResultSet r;
		String SQL1 = "Insert into [Historial Partidas] ([Id Partida], [Id Jugador1], [Id Jugador2], Resultado, fechaHora, Movimientos) values (?, ?, ?, ?, ?, ?)";
		String SQL2 = "Select [Partidas Jugadas], [Partidas Ganadas], [Partidas Perdidas], [Partidas Tablas], Puntuacion from Jugador WHERE nombreUsuario = ?;";
		String SQL3 = "Select [Partidas Jugadas], [Partidas Ganadas], [Partidas Perdidas], [Partidas Tablas], Puntuacion from Jugador WHERE nombreUsuario = ?;";
		String SQL4 = "Update Jugador SET [Partidas Jugadas] = ?, [Partidas Ganadas] = ?, [Partidas Perdidas] = ?, [Partidas Tablas] = ?, Puntuacion = ? WHERE nombreUsuario = ?;";
		String SQL5 = "Update Jugador SET [Partidas Jugadas] = ?, [Partidas Ganadas] = ?, [Partidas Perdidas] = ?, [Partidas Tablas] = ?, Puntuacion = ? WHERE nombreUsuario = ?;";
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL1);
			p.setInt(1, idPart);
			p.setString(2, id1);
			p.setString(3, id2);
			p.setInt(4, resul);
			p.setString(5, Fecha);
			p.setString(6, movimientos);
			p.executeUpdate();
			
			// Se actualiza la informacin del jugador1
			p = bd.prepareStatement(SQL2);
			p.setString(1, id1);
			r = p.executeQuery();
			if (r.next()) {
				PJ1 = r.getInt(1);
				PG1 = r.getInt(2);
				PP1 = r.getInt(3);
				PT1 = r.getInt(4);
				P1 = r.getInt(5);
			}
			
			// Se actualiza la informacin del jugador2
			p = bd.prepareStatement(SQL3);
			p.setString(1, id2);
			r = p.executeQuery();
			if (r.next()) {
				PJ2 = r.getInt(1);
				PG2 = r.getInt(2);
				PP2 = r.getInt(3);
				PT2 = r.getInt(4);
				P2 = r.getInt(5);
			}
			
			// Se actualizan los valores
			PJ1++;
			PJ2++;
			if (resul == 1) {  // Gan el jugador1
				PG1++;
				PP2++;
			}
			else {
				if (resul == 2) {  // Gan el jugador2
					PP1++;
					PG2++;
				}
				else {  // Hubo tablas
					PT1++;
					PT2++;
				}
			}
			
			// Se actualiza la puntuacin
			P1 = PG1*3 + PT1*1;
			P2 = PG2*3 + PT1*1;
			
			// Se modifican los valores del jugador1 en la base de datos
			p = bd.prepareStatement(SQL4);
			p.setInt(1, PJ1);
			p.setInt(2, PG1);
			p.setInt(3, PP1);
			p.setInt(4, PT1);
			p.setInt(5, P1);
			p.setString(6, id1);
			p.executeUpdate();
			
			// Se modifican los valores del jugador2 en la base de datos
			p = bd.prepareStatement(SQL5);
			p.setInt(1, PJ2);
			p.setInt(2, PG2);
			p.setInt(3, PP2);
			p.setInt(4, PT2);
			p.setInt(5, P2);
			p.setString(6, id2);
			p.executeUpdate();
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally{
			try {
				bd.close();
			} catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
	} /* Fin mtodo insertaPartida */
	
	
	/**
	 * Obtiene los datos clasificatorios del jugador.
	 * @return   Devuelve una matriz de cinco componentes:
	 *           0 - partidas jugadas
	 *           1 - partidas ganadas
	 *           2 - partidas perdidas
	 *           3 - partidas en tablas
	 *           4 - puntuacin
	 */
	public static String[] getClasificacion(String sUsuario) {
		String [] asDatos = new String[5];
		PreparedStatement p = null;
		Connection bd = null;
		ResultSet r;
		String SQL = "SELECT [Partidas Jugadas], [Partidas Ganadas], [Partidas Perdidas], [Partidas Tablas], Puntuacion FROM Jugador WHERE nombreUsuario = ?;";
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL);
			p.setString(1, sUsuario);
			r = p.executeQuery();
			if (r.next()) {
				asDatos[0] = ""+r.getInt(1);
				asDatos[1] = ""+r.getInt(2);
				asDatos[2] = ""+r.getInt(3);
				asDatos[3] = ""+r.getInt(4);
				asDatos[4] = ""+r.getInt(5);
			}
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		return asDatos;
	} /* Fin mtodo getClasificacion */
	
} /* Fin clase Metodos */
